@@ -24,6 +24,7 @@ from .engine_intf import GenericStreamWrapper, BytesIOWrapper
 from .proto_handlers import load_protocol_handler
 from .utils.connect import connect_to_database, dataframe_to_sql, db_conn_flavor
 from .utils.dbconn import sql_enquote_id
+from .utils.df_utils import CloseAfterIteration, load_dataframe_from_sql, LoadStrategy
 from .utils.file_utils import write_stream_with_read_on_close, connect_to_filesystem, adapt_fsspec_storage_options
 from .utils.format_sniffer import find_url_extension, detect_format
 from .utils.nosql import connect_to_nosql
@@ -823,15 +824,19 @@ class DSLibrary(object):
             nosql_collection - name of a NoSQL collection to fully load
             run_data - True to read using open_run_data()
             dask - True to load into a dask dataframe, or an integer to load into dask if file size is >= to threshold.
+            sniff_only - True to do all format analysis and return a LoadStrategy, rather than a dataframe
 
         :param resource_name:   The name of an input resource, or of a local file, or the URI of an external resource.
         """
+        sniff_only = kwargs.pop("sniff_only", None)
         # look up pre-supplied data
         # do translation of named resources
         if isinstance(resource_name, str):
             # check for specifically supplied data
             supplied = (self._spec.get("data") or {}).get(resource_name)
             if supplied is not None and hasattr(supplied, "columns"):
+                if sniff_only:
+                    return LoadStrategy(predefined=supplied)
                 return supplied
             # map requested resource (resource_name) through input spec and override defaults (kwargs)
             uri, open_args = self._xlt_resource(resource_name, **kwargs)
@@ -859,23 +864,16 @@ class DSLibrary(object):
             if open_args.get("sql") or open_args.get("sql_table"):
                 sql = open_args.pop("sql", None)
                 table = open_args.pop("sql_table", None)
+                if sniff_only:
+                    open_args["sniff_only"] = True
                 db_conn = self.get_sql_connection(resource_name, **open_args)
                 if table:
                     flavor = db_conn_flavor(db_conn)
                     sql = f"SELECT * from {sql_enquote_id(table, dotted=True, mode=flavor)}"
-                try:
-                    c = db_conn.cursor()
-                    c.execute(sql)
-                    col_names = [col[0] for col in c.description]
-                    df = pandas.DataFrame(data=c, columns=col_names)
-                    if use_dask:
-                        mem_size = df.memory_usage().sum()
-                        if mem_size > use_dask:
-                            partition_size = max(use_dask, 40000000)
-                            df = dask.dataframe.from_pandas(df, npartitions=min(mem_size//partition_size, 2))
-                finally:
-                    db_conn.close()
-                return df
+                if sniff_only:
+                    connect_strategy = db_conn
+                    return LoadStrategy(opener=connect_strategy.opener, open_args=connect_strategy.open_args, read_method=load_dataframe_from_sql, use_dask=use_dask, sql=sql)
+                return load_dataframe_from_sql(db_conn, sql, use_dask)
             # detect NoSQL
             if open_args.get("nosql_collection"):
                 collection = open_args.pop("nosql_collection")
@@ -910,8 +908,11 @@ class DSLibrary(object):
             use_opener = open_existing_stream
         else:
             # list, dict, etc.
-            # FIXME what to do with all the other arguments?  dask, at least could be applied
-            return pandas.DataFrame(resource_name)
+            # FIXME what to do with all the other arguments?  use_dask, at least could be applied
+            df = pandas.DataFrame(resource_name)
+            if sniff_only:
+                return LoadStrategy(predefined=df)
+            return df
         # sample-taker
         def sampler():
             sample_size = open_args.pop("sample_size", None) or 30000
@@ -962,6 +963,13 @@ class DSLibrary(object):
                     read_args["assume_missing"] = True
         else:
             read_method = PANDAS_READ_FORMATS.get(format)
+        # sniffing only?
+        if sniff_only:
+            return LoadStrategy(
+                format=format, read_args=read_args, use_dask=use_dask, read_method=read_method,
+                uri=uri, open_args=open_args, opener=use_opener
+            )
+        # all preparation is done, now we can actually read stuff
         if read_method:
             if use_dask:
                 open_args = adapt_fsspec_storage_options(open_args)
@@ -971,40 +979,6 @@ class DSLibrary(object):
                     return read_method(uri + "/*", **open_args, **read_args)
             # when returning a chunk stream we need to keep the stream open until iteration completes
             if format_info.read_args.get("chunksize"):
-                class CloseAfterIteration(object):
-                    def __init__(self, stream, iterator):
-                        self.stream = stream
-                        self.iterator = iterator
-                        self.closed = False
-
-                    def __iter__(self):
-                        return self
-
-                    def __next__(self):
-                        try:
-                            return next(self.iterator)
-                        except StopIteration:
-                            self.close()
-                            raise
-
-                    def close(self):
-                        if not self.closed:
-                            self.stream.close()
-                            self.closed = True
-
-                    def tell(self):
-                        if not hasattr(self.stream, "tell"):
-                            return
-                        return self.stream.tell()
-
-                    def get_chunk(self, size=None):
-                        try:
-                            if hasattr(self.iterator, "get_chunk"):
-                                return self.iterator.get_chunk(size)
-                        except StopIteration:
-                            self.close()
-                            raise
-
                 stream = use_opener(uri, mode='rb', **open_args)
                 iterator = self._open_from_stream(stream, read_method, read_args, format)
                 return CloseAfterIteration(stream, iterator)
@@ -1165,6 +1139,31 @@ def _coerce_to_dataframe(content):
         raise NotImplementedError(f"cannot convert {type(content)} to dataframe")
 
 
+def _read_hdf(filename: str, **kwargs):
+    """
+    Not all HDF files can stream.  This method falls back to loading the whole dataframe at once, but presenting
+    it as a chunk stream.
+    """
+    try:
+        return pandas.read_hdf(filename, **kwargs)
+    except TypeError as err:
+        if kwargs.get("chunksize") and "chunksize" in str(err):
+            kwargs.pop("chunksize")
+            df = pandas.read_hdf(filename, **kwargs)
+            class ChunkOne(object):
+                def __init__(self):
+                    self.done = False
+                def get_chunk(self, size=None):
+                    if self.done:
+                        raise StopIteration
+                    self.done = True
+                    return df
+                def close(self):
+                    pass
+            return ChunkOne()
+        raise
+
+
 PANDAS_READ_FORMATS = {
     "csv": pandas.read_csv,
     # TODO should we use json_normalize() or read_json() here, or add a switch?
@@ -1174,7 +1173,7 @@ PANDAS_READ_FORMATS = {
     "parquet": pandas.read_parquet,
     "xlsx": pandas.read_excel,
     "xls": pandas.read_excel,
-    "hdf": pandas.read_hdf,
+    "hdf": _read_hdf,
     "fwf": pandas.read_fwf
 }
 
