@@ -42,7 +42,8 @@ VALID_OPTIONS = {
     "csv": VALID_CSV_OPTIONS,
     "json": VALID_JSON_OPTIONS,
     "hdf": VALID_HDF_OPTIONS,
-    "xlsx": VALID_EXCEL_OPTIONS
+    "xlsx": VALID_EXCEL_OPTIONS,
+    "parquet": VALID_PARQUET_OPTIONS
 }
 
 
@@ -78,6 +79,8 @@ def detect_format(
         "csv": ("csv", {}),
         "json": ("json", {}),
         "yml": ("yaml", {}),
+        "pq": ("parquet", {}),
+        "h5": ("hdf", {}),
         "tab": ("csv", {"delimiter": "\t"}),
         "tsv": ("csv", {"delimiter": "\t"}),
         "txt": ("csv", {"names": ["line"]})
